﻿/*jslint node: true */
"use strict";

var sphereGeometry = {
  vertexPositions: [],
  vertexNormals: [],
  vertexTextureCoords: [],
  indices: [],
  colors: [],
  emissiveColors: [],
  diffuseColors: [],
  specularColors: [],
};

function makePlane(rows) {
  var step = 1 / (rows - 1);
  var i = 0, s = 1 / 2;
  var t = [];
  for (var x = 0; x < rows; x++) {
    for (var y = 0; y < rows; y++) {
      var x1, y1, z1, r;
      x1 = -s + x * step;
      y1 = -s + y * step;
      z1 = -s;
      r = Math.sqrt(x1 * x1 + y1 * y1 + z1 * z1);

      x1 /= r;
      y1 /= r;
      z1 /= r;

      t[i++] = x1;
      t[i++] = y1;
      t[i++] = z1;
    }
  }
  return t;
}

function initSphereGeometry(rows) {
  rows = rows || 9;

  var v = makePlane(rows);

  // determine quadrant limits
  var q1 = rows / 4;
  var q3 = (q1 - 1) * 3;
  q1++;

  for (var i = 0; i < rows - 1; i++) {
    for (var j = 0; j < rows - 1; j++) {
      if ((i < q1 && j < q1) || (i > q3 && j > q3)) {
        sphereGeometry.indices.push(i + (j + 1) * rows + 1);
        sphereGeometry.indices.push(i + j * rows);
        sphereGeometry.indices.push(i + j * rows + 1);

        sphereGeometry.indices.push(i + (j + 1) * rows + 1);
        sphereGeometry.indices.push(i + (j + 1) * rows);
        sphereGeometry.indices.push(i + j * rows);
      } else {
        sphereGeometry.indices.push(i + j * rows);
        sphereGeometry.indices.push(i + j * rows + 1);
        sphereGeometry.indices.push(i + (j + 1) * rows);

        sphereGeometry.indices.push(i + (j + 1) * rows);
        sphereGeometry.indices.push(i + j * rows + 1);
        sphereGeometry.indices.push(i + (j + 1) * rows + 1);
      }
    }
  }

  var l = v.length;
  for (var i = 0; i < l; i += 3) {
    v.push(v[i + 1]);
    v.push(v[i]);
    v.push(-v[i + 2]);
  }

  appendIndices(sphereGeometry.indices.length, l / 3);

  // copy and append vertices rotated through 90 degrees
  l = v.length;
  for (var i = 0; i < l; i += 3) {
    v.push(v[i + 2]);
    v.push(v[i]);
    v.push(v[i + 1]);
  }
  appendIndices(sphereGeometry.indices.length, l / 3);

  for (var i = 0; i < l; i += 3) {
    v.push(v[i]);
    v.push(v[i + 2]);
    v.push(-v[i + 1]);
  }
  appendIndices(sphereGeometry.indices.length/2, 2*l / 3);

  sphereGeometry.vertexPositions = v;
  for (var i = 0; i < sphereGeometry.vertexPositions.length/3; i++) {
    sphereGeometry.colors.push(1); sphereGeometry.colors.push(1); sphereGeometry.colors.push(0); sphereGeometry.colors.push(1);

    sphereGeometry.vertexNormals.push(sphereGeometry.vertexPositions[3 * i]);
    sphereGeometry.vertexNormals.push(sphereGeometry.vertexPositions[3 * i + 1]);
    sphereGeometry.vertexNormals.push(sphereGeometry.vertexPositions[3 * i + 2]);

    sphereGeometry.vertexTextureCoords.push(0); sphereGeometry.vertexTextureCoords.push(0);
  }

  return sphereGeometry;
}

function appendIndices(limit, offset) {
  for (var i = 0; i < limit; i++) {
    sphereGeometry.indices.push(sphereGeometry.indices[i] + offset);
  }
}
